# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ...types.v1 import plan_list_params, plan_list_charges_params, plan_list_customers_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.plan_list_response import PlanListResponse
from ...types.v1.plan_get_details_response import PlanGetDetailsResponse
from ...types.v1.plan_list_charges_response import PlanListChargesResponse
from ...types.v1.plan_list_customers_response import PlanListCustomersResponse

__all__ = ["PlansResource", "AsyncPlansResource"]


class PlansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return PlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return PlansResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[PlanListResponse]:
        """List all available plans.

        This is a Plans (deprecated) endpoint. New clients
        should implement using Contracts.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/plans",
            page=SyncCursorPage[PlanListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_params.PlanListParams,
                ),
            ),
            model=PlanListResponse,
        )

    def get_details(
        self,
        *,
        plan_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanGetDetailsResponse:
        """Fetch high level details of a specific plan.

        This is a Plans (deprecated)
        endpoint. New clients should implement using Contracts.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._get(
            f"/v1/planDetails/{plan_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGetDetailsResponse,
        )

    def list_charges(
        self,
        *,
        plan_id: str,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[PlanListChargesResponse]:
        """Fetches a list of charges of a specific plan.

        This is a Plans (deprecated)
        endpoint. New clients should implement using Contracts.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._get_api_list(
            f"/v1/planDetails/{plan_id}/charges",
            page=SyncCursorPage[PlanListChargesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_charges_params.PlanListChargesParams,
                ),
            ),
            model=PlanListChargesResponse,
        )

    def list_customers(
        self,
        *,
        plan_id: str,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        status: Literal["all", "active", "ended", "upcoming"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[PlanListCustomersResponse]:
        """
        Fetches a list of customers on a specific plan (by default, only currently
        active plans are included). This is a Plans (deprecated) endpoint. New clients
        should implement using Contracts.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          status: Status of customers on a given plan. Defaults to `active`.

              - `all` - Return current, past, and upcoming customers of the plan.
              - `active` - Return current customers of the plan.
              - `ended` - Return past customers of the plan.
              - `upcoming` - Return upcoming customers of the plan.

              Multiple statuses can be OR'd together using commas, e.g. `active,ended`.
              **Note:** `ended,upcoming` combination is not yet supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._get_api_list(
            f"/v1/planDetails/{plan_id}/customers",
            page=SyncCursorPage[PlanListCustomersResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                        "status": status,
                    },
                    plan_list_customers_params.PlanListCustomersParams,
                ),
            ),
            model=PlanListCustomersResponse,
        )


class AsyncPlansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncPlansResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PlanListResponse, AsyncCursorPage[PlanListResponse]]:
        """List all available plans.

        This is a Plans (deprecated) endpoint. New clients
        should implement using Contracts.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/plans",
            page=AsyncCursorPage[PlanListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_params.PlanListParams,
                ),
            ),
            model=PlanListResponse,
        )

    async def get_details(
        self,
        *,
        plan_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanGetDetailsResponse:
        """Fetch high level details of a specific plan.

        This is a Plans (deprecated)
        endpoint. New clients should implement using Contracts.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return await self._get(
            f"/v1/planDetails/{plan_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGetDetailsResponse,
        )

    def list_charges(
        self,
        *,
        plan_id: str,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PlanListChargesResponse, AsyncCursorPage[PlanListChargesResponse]]:
        """Fetches a list of charges of a specific plan.

        This is a Plans (deprecated)
        endpoint. New clients should implement using Contracts.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._get_api_list(
            f"/v1/planDetails/{plan_id}/charges",
            page=AsyncCursorPage[PlanListChargesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                    },
                    plan_list_charges_params.PlanListChargesParams,
                ),
            ),
            model=PlanListChargesResponse,
        )

    def list_customers(
        self,
        *,
        plan_id: str,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        status: Literal["all", "active", "ended", "upcoming"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PlanListCustomersResponse, AsyncCursorPage[PlanListCustomersResponse]]:
        """
        Fetches a list of customers on a specific plan (by default, only currently
        active plans are included). This is a Plans (deprecated) endpoint. New clients
        should implement using Contracts.

        Args:
          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          status: Status of customers on a given plan. Defaults to `active`.

              - `all` - Return current, past, and upcoming customers of the plan.
              - `active` - Return current customers of the plan.
              - `ended` - Return past customers of the plan.
              - `upcoming` - Return upcoming customers of the plan.

              Multiple statuses can be OR'd together using commas, e.g. `active,ended`.
              **Note:** `ended,upcoming` combination is not yet supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not plan_id:
            raise ValueError(f"Expected a non-empty value for `plan_id` but received {plan_id!r}")
        return self._get_api_list(
            f"/v1/planDetails/{plan_id}/customers",
            page=AsyncCursorPage[PlanListCustomersResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "next_page": next_page,
                        "status": status,
                    },
                    plan_list_customers_params.PlanListCustomersParams,
                ),
            ),
            model=PlanListCustomersResponse,
        )


class PlansResourceWithRawResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.list = to_raw_response_wrapper(
            plans.list,
        )
        self.get_details = to_raw_response_wrapper(
            plans.get_details,
        )
        self.list_charges = to_raw_response_wrapper(
            plans.list_charges,
        )
        self.list_customers = to_raw_response_wrapper(
            plans.list_customers,
        )


class AsyncPlansResourceWithRawResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.list = async_to_raw_response_wrapper(
            plans.list,
        )
        self.get_details = async_to_raw_response_wrapper(
            plans.get_details,
        )
        self.list_charges = async_to_raw_response_wrapper(
            plans.list_charges,
        )
        self.list_customers = async_to_raw_response_wrapper(
            plans.list_customers,
        )


class PlansResourceWithStreamingResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.list = to_streamed_response_wrapper(
            plans.list,
        )
        self.get_details = to_streamed_response_wrapper(
            plans.get_details,
        )
        self.list_charges = to_streamed_response_wrapper(
            plans.list_charges,
        )
        self.list_customers = to_streamed_response_wrapper(
            plans.list_customers,
        )


class AsyncPlansResourceWithStreamingResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.list = async_to_streamed_response_wrapper(
            plans.list,
        )
        self.get_details = async_to_streamed_response_wrapper(
            plans.get_details,
        )
        self.list_charges = async_to_streamed_response_wrapper(
            plans.list_charges,
        )
        self.list_customers = async_to_streamed_response_wrapper(
            plans.list_customers,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContractUpdateEndDateParams"]


class ContractUpdateEndDateParams(TypedDict, total=False):
    contract_id: Required[str]
    """ID of the contract to update"""

    customer_id: Required[str]
    """ID of the customer whose contract is to be updated"""

    allow_ending_before_finalized_invoice: bool
    """
    If true, allows setting the contract end date earlier than the end_timestamp of
    existing finalized invoices. Finalized invoices will be unchanged; if you want
    to incorporate the new end date, you can void and regenerate finalized usage
    invoices. Defaults to true.
    """

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp indicating when the contract will end (exclusive).

    If not provided, the contract will be updated to be open-ended.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AuditLogListParams"]


class AuditLogListParams(TypedDict, total=False):
    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp (exclusive). Cannot be used with 'next_page'."""

    limit: int
    """Max number of results that should be returned"""

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    resource_id: str
    """Optional parameter that can be used to filter which audit logs are returned.

    If you specify resource_id, you must also specify resource_type.
    """

    resource_type: str
    """Optional parameter that can be used to filter which audit logs are returned.

    If you specify resource_type, you must also specify resource_id.
    """

    sort: Literal["date_asc", "date_desc"]
    """Sort order by timestamp, e.g. date_asc or date_desc. Defaults to date_asc."""

    starting_on: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """RFC 3339 timestamp of the earliest audit log to return.

    Cannot be used with 'next_page'.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .contract_edit_params import ContractEditParams as ContractEditParams
from .contract_list_params import ContractListParams as ContractListParams
from .contract_edit_response import ContractEditResponse as ContractEditResponse
from .contract_list_response import ContractListResponse as ContractListResponse
from .contract_retrieve_params import ContractRetrieveParams as ContractRetrieveParams
from .contract_retrieve_response import ContractRetrieveResponse as ContractRetrieveResponse
from .contract_edit_commit_params import ContractEditCommitParams as ContractEditCommitParams
from .contract_edit_credit_params import ContractEditCreditParams as ContractEditCreditParams
from .contract_edit_commit_response import ContractEditCommitResponse as ContractEditCommitResponse
from .contract_edit_credit_response import ContractEditCreditResponse as ContractEditCreditResponse
from .contract_get_edit_history_params import ContractGetEditHistoryParams as ContractGetEditHistoryParams
from .contract_get_edit_history_response import ContractGetEditHistoryResponse as ContractGetEditHistoryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["PropertyFilter"]


class PropertyFilter(TypedDict, total=False):
    name: Required[str]
    """The name of the event property."""

    exists: bool
    """Determines whether the property must exist in the event.

    If true, only events with this property will pass the filter. If false, only
    events without this property will pass the filter. If null or omitted, the
    existence of the property is optional.
    """

    in_values: SequenceNotStr[str]
    """Specifies the allowed values for the property to match an event.

    An event will pass the filter only if its property value is included in this
    list. If undefined, all property values will pass the filter. Must be non-empty
    if present.
    """

    not_in_values: SequenceNotStr[str]
    """Specifies the values that prevent an event from matching the filter.

    An event will not pass the filter if its property value is included in this
    list. If null or empty, all property values will pass the filter. Must be
    non-empty if present.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .payment_gate_config import PaymentGateConfig
from .base_threshold_commit import BaseThresholdCommit
from .commit_specifier_input import CommitSpecifierInput

__all__ = ["PrepaidBalanceThresholdConfiguration", "Commit"]


class Commit(BaseThresholdCommit):
    applicable_product_ids: Optional[List[str]] = None
    """Which products the threshold commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    applicable_product_tags: Optional[List[str]] = None
    """Which tags the threshold commit applies to.

    If applicable_product_ids, applicable_product_tags or specifiers are not
    provided, the commit applies to all products.
    """

    specifiers: Optional[List[CommitSpecifierInput]] = None
    """
    List of filters that determine what kind of customer usage draws down a commit
    or credit. A customer's usage needs to meet the condition of at least one of the
    specifiers to contribute to a commit's or credit's drawdown. This field cannot
    be used together with `applicable_product_ids` or `applicable_product_tags`.
    """


class PrepaidBalanceThresholdConfiguration(BaseModel):
    commit: Commit

    is_enabled: bool
    """
    When set to false, the contract will not be evaluated against the
    threshold_amount. Toggling to true will result an immediate evaluation,
    regardless of prior state.
    """

    payment_gate_config: PaymentGateConfig

    recharge_to_amount: float
    """Specify the amount the balance should be recharged to."""

    threshold_amount: float
    """Specify the threshold amount for the contract.

    Each time the contract's prepaid balance lowers to this amount, a threshold
    charge will be initiated.
    """

    custom_credit_type_id: Optional[str] = None
    """
    If provided, the threshold, recharge-to amount, and the resulting threshold
    commit amount will be in terms of this credit type instead of the fiat currency.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel
from ...shared.credit_type_data import CreditTypeData

__all__ = [
    "CustomerAlert",
    "Alert",
    "AlertCustomFieldFilter",
    "AlertGroupKeyFilter",
    "AlertGroupValue",
    "AlertSeatFilter",
]


class AlertCustomFieldFilter(BaseModel):
    entity: Literal["Contract", "Commit", "ContractCredit"]

    key: str

    value: str


class AlertGroupKeyFilter(BaseModel):
    """
    Scopes threshold notification evaluation to a specific presentation group key on individual line items. Only present for spend notifications.
    """

    key: str

    value: str


class AlertGroupValue(BaseModel):
    key: str

    value: Optional[str] = None


class AlertSeatFilter(BaseModel):
    """Only present for low_remaining_seat_balance_reached notifications.

    The seat group key or seat group key-value pair the alert is scoped to.
    """

    seat_group_key: str
    """The seat group key (e.g., "seat_id", "user_id") that the alert is scoped to."""

    seat_group_value: Optional[str] = None
    """The seat group value that the alert is scoped to."""


class Alert(BaseModel):
    id: str
    """the Metronome ID of the threshold notification"""

    name: str
    """Name of the threshold notification"""

    status: Literal["enabled", "archived", "disabled"]
    """Status of the threshold notification"""

    threshold: float
    """Threshold value of the notification policy"""

    type: Literal[
        "low_credit_balance_reached",
        "spend_threshold_reached",
        "monthly_invoice_total_spend_threshold_reached",
        "low_remaining_days_in_plan_reached",
        "low_remaining_credit_percentage_reached",
        "usage_threshold_reached",
        "low_remaining_days_for_commit_segment_reached",
        "low_remaining_commit_balance_reached",
        "low_remaining_commit_percentage_reached",
        "low_remaining_days_for_contract_credit_segment_reached",
        "low_remaining_contract_credit_balance_reached",
        "low_remaining_contract_credit_percentage_reached",
        "low_remaining_contract_credit_and_commit_balance_reached",
        "low_remaining_seat_balance_reached",
        "invoice_total_reached",
    ]
    """Type of the threshold notification"""

    updated_at: datetime
    """Timestamp for when the threshold notification was last updated"""

    credit_grant_type_filters: Optional[List[str]] = None
    """
    An array of strings, representing a way to filter the credit grant this
    threshold notification applies to, by looking at the credit_grant_type field on
    the credit grant. This field is only defined for CreditPercentage and
    CreditBalance notifications
    """

    credit_type: Optional[CreditTypeData] = None

    custom_field_filters: Optional[List[AlertCustomFieldFilter]] = None
    """
    A list of custom field filters for notification types that support advanced
    filtering
    """

    group_key_filter: Optional[AlertGroupKeyFilter] = None
    """
    Scopes threshold notification evaluation to a specific presentation group key on
    individual line items. Only present for spend notifications.
    """

    group_values: Optional[List[AlertGroupValue]] = None
    """Only present for `spend_threshold_reached` notifications.

    Scope notification to a specific group key on individual line items.
    """

    invoice_types_filter: Optional[List[str]] = None
    """Only supported for invoice_total_reached threshold notifications.

    A list of invoice types to evaluate.
    """

    seat_filter: Optional[AlertSeatFilter] = None
    """Only present for low_remaining_seat_balance_reached notifications.

    The seat group key or seat group key-value pair the alert is scoped to.
    """

    uniqueness_key: Optional[str] = None
    """Prevents the creation of duplicates.

    If a request to create a record is made with a previously used uniqueness key, a
    new record will not be created and the request will fail with a 409 error.
    """


class CustomerAlert(BaseModel):
    alert: Alert

    customer_status: Optional[Literal["ok", "in_alarm", "evaluating"]] = None
    """The status of the threshold notification.

    If the notification is archived, null will be returned.
    """

    triggered_by: Optional[str] = None
    """If present, indicates the reason the threshold notification was triggered."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from ...._models import BaseModel
from ...shared.credit_type_data import CreditTypeData

__all__ = ["RateCardListResponse", "Alias", "CreditTypeConversion"]


class Alias(BaseModel):
    name: str

    ending_before: Optional[datetime] = None

    starting_at: Optional[datetime] = None


class CreditTypeConversion(BaseModel):
    custom_credit_type: CreditTypeData

    fiat_per_custom_credit: str


class RateCardListResponse(BaseModel):
    id: str

    created_at: datetime

    created_by: str

    name: str

    aliases: Optional[List[Alias]] = None

    credit_type_conversions: Optional[List[CreditTypeConversion]] = None

    custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    description: Optional[str] = None

    fiat_credit_type: Optional[CreditTypeData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ....._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.v1.contracts.rate_cards import product_order_set_params, product_order_update_params
from .....types.v1.contracts.rate_cards.product_order_set_response import ProductOrderSetResponse
from .....types.v1.contracts.rate_cards.product_order_update_response import ProductOrderUpdateResponse

__all__ = ["ProductOrdersResource", "AsyncProductOrdersResource"]


class ProductOrdersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProductOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return ProductOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProductOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return ProductOrdersResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        product_moves: Iterable[product_order_update_params.ProductMove],
        rate_card_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductOrderUpdateResponse:
        """
        The ordering of products on a rate card determines the order in which the
        products will appear on customers' invoices. Use this endpoint to set the order
        of specific products on the rate card by moving them relative to their current
        location.

        Args:
          rate_card_id: ID of the rate card to update

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/moveRateCardProducts",
            body=maybe_transform(
                {
                    "product_moves": product_moves,
                    "rate_card_id": rate_card_id,
                },
                product_order_update_params.ProductOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductOrderUpdateResponse,
        )

    def set(
        self,
        *,
        product_order: SequenceNotStr[str],
        rate_card_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductOrderSetResponse:
        """
        The ordering of products on a rate card determines the order in which the
        products will appear on customers' invoices. Use this endpoint to set the order
        of products on the rate card.

        Args:
          rate_card_id: ID of the rate card to update

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/contract-pricing/rate-cards/setRateCardProductsOrder",
            body=maybe_transform(
                {
                    "product_order": product_order,
                    "rate_card_id": rate_card_id,
                },
                product_order_set_params.ProductOrderSetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductOrderSetResponse,
        )


class AsyncProductOrdersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProductOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProductOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProductOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncProductOrdersResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        product_moves: Iterable[product_order_update_params.ProductMove],
        rate_card_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductOrderUpdateResponse:
        """
        The ordering of products on a rate card determines the order in which the
        products will appear on customers' invoices. Use this endpoint to set the order
        of specific products on the rate card by moving them relative to their current
        location.

        Args:
          rate_card_id: ID of the rate card to update

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/moveRateCardProducts",
            body=await async_maybe_transform(
                {
                    "product_moves": product_moves,
                    "rate_card_id": rate_card_id,
                },
                product_order_update_params.ProductOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductOrderUpdateResponse,
        )

    async def set(
        self,
        *,
        product_order: SequenceNotStr[str],
        rate_card_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProductOrderSetResponse:
        """
        The ordering of products on a rate card determines the order in which the
        products will appear on customers' invoices. Use this endpoint to set the order
        of products on the rate card.

        Args:
          rate_card_id: ID of the rate card to update

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/contract-pricing/rate-cards/setRateCardProductsOrder",
            body=await async_maybe_transform(
                {
                    "product_order": product_order,
                    "rate_card_id": rate_card_id,
                },
                product_order_set_params.ProductOrderSetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProductOrderSetResponse,
        )


class ProductOrdersResourceWithRawResponse:
    def __init__(self, product_orders: ProductOrdersResource) -> None:
        self._product_orders = product_orders

        self.update = to_raw_response_wrapper(
            product_orders.update,
        )
        self.set = to_raw_response_wrapper(
            product_orders.set,
        )


class AsyncProductOrdersResourceWithRawResponse:
    def __init__(self, product_orders: AsyncProductOrdersResource) -> None:
        self._product_orders = product_orders

        self.update = async_to_raw_response_wrapper(
            product_orders.update,
        )
        self.set = async_to_raw_response_wrapper(
            product_orders.set,
        )


class ProductOrdersResourceWithStreamingResponse:
    def __init__(self, product_orders: ProductOrdersResource) -> None:
        self._product_orders = product_orders

        self.update = to_streamed_response_wrapper(
            product_orders.update,
        )
        self.set = to_streamed_response_wrapper(
            product_orders.set,
        )


class AsyncProductOrdersResourceWithStreamingResponse:
    def __init__(self, product_orders: AsyncProductOrdersResource) -> None:
        self._product_orders = product_orders

        self.update = async_to_streamed_response_wrapper(
            product_orders.update,
        )
        self.set = async_to_streamed_response_wrapper(
            product_orders.set,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from ..shared_params.property_filter import PropertyFilter
from ..shared_params.event_type_filter import EventTypeFilter

__all__ = ["BillableMetricCreateParams"]


class BillableMetricCreateParams(TypedDict, total=False):
    name: Required[str]
    """The display name of the billable metric."""

    aggregation_key: str
    """Specifies the type of aggregation performed on matching events.

    Required if `sql` is not provided.
    """

    aggregation_type: Literal["COUNT", "LATEST", "MAX", "SUM", "UNIQUE"]
    """Specifies the type of aggregation performed on matching events."""

    custom_fields: Dict[str, str]
    """Custom fields to attach to the billable metric."""

    event_type_filter: EventTypeFilter
    """An optional filtering rule to match the 'event_type' property of an event."""

    group_keys: Iterable[SequenceNotStr[str]]
    """Property names that are used to group usage costs on an invoice.

    Each entry represents a set of properties used to slice events into distinct
    buckets.
    """

    property_filters: Iterable[PropertyFilter]
    """A list of filters to match events to this billable metric.

    Each filter defines a rule on an event property. All rules must pass for the
    event to match the billable metric.
    """

    sql: str
    """The SQL query associated with the billable metric.

    This field is mutually exclusive with aggregation_type, event_type_filter,
    property_filters, aggregation_key, and group_keys. If provided, these other
    fields must be omitted.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PaymentGateConfig", "PrecalculatedTaxConfig", "StripeConfig"]


class PrecalculatedTaxConfig(BaseModel):
    """Only applicable if using PRECALCULATED as your tax type."""

    tax_amount: float
    """Amount of tax to be applied.

    This should be in the same currency and denomination as the commit's invoice
    schedule
    """

    tax_name: Optional[str] = None
    """Name of the tax to be applied.

    This may be used in an invoice line item description.
    """


class StripeConfig(BaseModel):
    """Only applicable if using STRIPE as your payment gate type."""

    payment_type: Literal["INVOICE", "PAYMENT_INTENT"]
    """If left blank, will default to INVOICE"""

    invoice_metadata: Optional[Dict[str, str]] = None
    """Metadata to be added to the Stripe invoice.

    Only applicable if using INVOICE as your payment type.
    """


class PaymentGateConfig(BaseModel):
    payment_gate_type: Literal["NONE", "STRIPE", "EXTERNAL"]
    """Gate access to the commit balance based on successful collection of payment.

    Select STRIPE for Metronome to facilitate payment via Stripe. Select EXTERNAL to
    facilitate payment using your own payment integration. Select NONE if you do not
    wish to payment gate the commit balance.
    """

    precalculated_tax_config: Optional[PrecalculatedTaxConfig] = None
    """Only applicable if using PRECALCULATED as your tax type."""

    stripe_config: Optional[StripeConfig] = None
    """Only applicable if using STRIPE as your payment gate type."""

    tax_type: Optional[Literal["NONE", "STRIPE", "ANROK", "AVALARA", "PRECALCULATED"]] = None
    """Stripe tax is only supported for Stripe payment gateway.

    Select NONE if you do not wish Metronome to calculate tax on your behalf.
    Leaving this field blank will default to NONE.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import payment_list_params, payment_cancel_params, payment_attempt_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncBodyCursorPage, AsyncBodyCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.payment import Payment
from ...types.v1.payment_status import PaymentStatus
from ...types.v1.payment_cancel_response import PaymentCancelResponse
from ...types.v1.payment_attempt_response import PaymentAttemptResponse

__all__ = ["PaymentsResource", "AsyncPaymentsResource"]


class PaymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return PaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return PaymentsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        statuses: List[PaymentStatus] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncBodyCursorPage[Payment]:
        """
        Fetch all payment attempts for the given invoice.

        Args:
          limit: The maximum number of payments to return. Defaults to 25.

          next_page: The next page token from a previous response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/payments/list",
            page=SyncBodyCursorPage[Payment],
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "invoice_id": invoice_id,
                    "limit": limit,
                    "next_page": next_page,
                    "statuses": statuses,
                },
                payment_list_params.PaymentListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Payment,
            method="post",
        )

    def attempt(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentAttemptResponse:
        """
        Trigger a new attempt by canceling any existing attempts for this invoice and
        creating a new Payment. This will trigger another attempt to charge the
        Customer's configured Payment Gateway. Payment can only be attempted if all of
        the following are true:

        - The Metronome Invoice is finalized
        - PLG Invoicing is configured for the Customer
        - You cannot attempt payments for invoices that have already been `paid` or
          `voided`.

        Attempting to payment on an ineligible Invoice or Customer will result in a
        `400` response.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/payments/attempt",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "invoice_id": invoice_id,
                },
                payment_attempt_params.PaymentAttemptParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentAttemptResponse,
        )

    def cancel(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentCancelResponse:
        """
        Cancel an existing payment attempt for an invoice.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/payments/cancel",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "invoice_id": invoice_id,
                },
                payment_cancel_params.PaymentCancelParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentCancelResponse,
        )


class AsyncPaymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncPaymentsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        statuses: List[PaymentStatus] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Payment, AsyncBodyCursorPage[Payment]]:
        """
        Fetch all payment attempts for the given invoice.

        Args:
          limit: The maximum number of payments to return. Defaults to 25.

          next_page: The next page token from a previous response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/payments/list",
            page=AsyncBodyCursorPage[Payment],
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "invoice_id": invoice_id,
                    "limit": limit,
                    "next_page": next_page,
                    "statuses": statuses,
                },
                payment_list_params.PaymentListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Payment,
            method="post",
        )

    async def attempt(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentAttemptResponse:
        """
        Trigger a new attempt by canceling any existing attempts for this invoice and
        creating a new Payment. This will trigger another attempt to charge the
        Customer's configured Payment Gateway. Payment can only be attempted if all of
        the following are true:

        - The Metronome Invoice is finalized
        - PLG Invoicing is configured for the Customer
        - You cannot attempt payments for invoices that have already been `paid` or
          `voided`.

        Attempting to payment on an ineligible Invoice or Customer will result in a
        `400` response.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/payments/attempt",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "invoice_id": invoice_id,
                },
                payment_attempt_params.PaymentAttemptParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentAttemptResponse,
        )

    async def cancel(
        self,
        *,
        customer_id: str,
        invoice_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentCancelResponse:
        """
        Cancel an existing payment attempt for an invoice.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/payments/cancel",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "invoice_id": invoice_id,
                },
                payment_cancel_params.PaymentCancelParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentCancelResponse,
        )


class PaymentsResourceWithRawResponse:
    def __init__(self, payments: PaymentsResource) -> None:
        self._payments = payments

        self.list = to_raw_response_wrapper(
            payments.list,
        )
        self.attempt = to_raw_response_wrapper(
            payments.attempt,
        )
        self.cancel = to_raw_response_wrapper(
            payments.cancel,
        )


class AsyncPaymentsResourceWithRawResponse:
    def __init__(self, payments: AsyncPaymentsResource) -> None:
        self._payments = payments

        self.list = async_to_raw_response_wrapper(
            payments.list,
        )
        self.attempt = async_to_raw_response_wrapper(
            payments.attempt,
        )
        self.cancel = async_to_raw_response_wrapper(
            payments.cancel,
        )


class PaymentsResourceWithStreamingResponse:
    def __init__(self, payments: PaymentsResource) -> None:
        self._payments = payments

        self.list = to_streamed_response_wrapper(
            payments.list,
        )
        self.attempt = to_streamed_response_wrapper(
            payments.attempt,
        )
        self.cancel = to_streamed_response_wrapper(
            payments.cancel,
        )


class AsyncPaymentsResourceWithStreamingResponse:
    def __init__(self, payments: AsyncPaymentsResource) -> None:
        self._payments = payments

        self.list = async_to_streamed_response_wrapper(
            payments.list,
        )
        self.attempt = async_to_streamed_response_wrapper(
            payments.attempt,
        )
        self.cancel = async_to_streamed_response_wrapper(
            payments.cancel,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .plan_detail import PlanDetail

__all__ = ["PlanGetDetailsResponse"]


class PlanGetDetailsResponse(BaseModel):
    data: PlanDetail



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Tier"]


class Tier(BaseModel):
    price: float

    size: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.service_list_response import ServiceListResponse

__all__ = ["ServicesResource", "AsyncServicesResource"]


class ServicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return ServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return ServicesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceListResponse:
        """
        Gets Metronome's service registry with associated IP addresses for security
        allowlisting and firewall configuration. Use this endpoint to maintain an
        up-to-date list of IPs that your systems should trust for Metronome
        communications. Returns service names and their current IP ranges, with new IPs
        typically appearing 30+ days before first use to ensure smooth allowlist
        updates.
        """
        return self._get(
            "/v1/services",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ServiceListResponse,
        )


class AsyncServicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncServicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncServicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncServicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncServicesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceListResponse:
        """
        Gets Metronome's service registry with associated IP addresses for security
        allowlisting and firewall configuration. Use this endpoint to maintain an
        up-to-date list of IPs that your systems should trust for Metronome
        communications. Returns service names and their current IP ranges, with new IPs
        typically appearing 30+ days before first use to ensure smooth allowlist
        updates.
        """
        return await self._get(
            "/v1/services",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ServiceListResponse,
        )


class ServicesResourceWithRawResponse:
    def __init__(self, services: ServicesResource) -> None:
        self._services = services

        self.list = to_raw_response_wrapper(
            services.list,
        )


class AsyncServicesResourceWithRawResponse:
    def __init__(self, services: AsyncServicesResource) -> None:
        self._services = services

        self.list = async_to_raw_response_wrapper(
            services.list,
        )


class ServicesResourceWithStreamingResponse:
    def __init__(self, services: ServicesResource) -> None:
        self._services = services

        self.list = to_streamed_response_wrapper(
            services.list,
        )


class AsyncServicesResourceWithStreamingResponse:
    def __init__(self, services: AsyncServicesResource) -> None:
        self._services = services

        self.list = async_to_streamed_response_wrapper(
            services.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ...types.v1 import audit_log_list_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.audit_log_list_response import AuditLogListResponse

__all__ = ["AuditLogsResource", "AsyncAuditLogsResource"]


class AuditLogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AuditLogsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_before: Union[str, datetime] | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        resource_id: str | Omit = omit,
        resource_type: str | Omit = omit,
        sort: Literal["date_asc", "date_desc"] | Omit = omit,
        starting_on: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[AuditLogListResponse]:
        """
        Get a comprehensive audit trail of all operations performed in your Metronome
        account, whether initiated through the API, web interface, or automated
        processes. This endpoint provides detailed logs of who did what and when,
        enabling compliance reporting, security monitoring, and operational
        troubleshooting across all interaction channels.

        ### Use this endpoint to:

        - Monitor all account activity for security and compliance purposes
        - Track configuration changes regardless of source (API, UI, or system)
        - Investigate issues by reviewing historical operations

        ### Key response fields:

        An array of AuditLog objects containing:

        - id: Unique identifier for the audit log entry
        - timestamp: When the action occurred (RFC 3339 format)
        - actor: Information about who performed the action
        - request: Details including request ID, IP address, and user agent
        - `resource_type`: The type of resource affected (e.g., customer, contract,
          invoice)
        - `resource_id`: The specific resource identifier
        - `action`: The operation performed
        - `next_page`: Cursor for continuous log retrieval

        ### Usage guidelines:

        - Continuous retrieval: The next_page token enables uninterrupted log
          streaming—save it between requests to ensure no logs are missed
        - Empty responses: An empty data array means no new logs yet; continue polling
          with the same next_page token
        - Date filtering:
          - `starting_on`: Earliest logs to return (inclusive)
          - `ending_before`: Latest logs to return (exclusive)
          - Cannot be used with `next_page`
        - Resource filtering: Must specify both `resource_type` and `resource_id`
          together
        - Sort order: Default is `date_asc`; use `date_desc` for newest first

        Args:
          ending_before: RFC 3339 timestamp (exclusive). Cannot be used with 'next_page'.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          resource_id: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_id, you must also specify resource_type.

          resource_type: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_type, you must also specify resource_id.

          sort: Sort order by timestamp, e.g. date_asc or date_desc. Defaults to date_asc.

          starting_on: RFC 3339 timestamp of the earliest audit log to return. Cannot be used with
              'next_page'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/auditLogs",
            page=SyncCursorPage[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "next_page": next_page,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "sort": sort,
                        "starting_on": starting_on,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
            ),
            model=AuditLogListResponse,
        )


class AsyncAuditLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncAuditLogsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_before: Union[str, datetime] | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        resource_id: str | Omit = omit,
        resource_type: str | Omit = omit,
        sort: Literal["date_asc", "date_desc"] | Omit = omit,
        starting_on: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuditLogListResponse, AsyncCursorPage[AuditLogListResponse]]:
        """
        Get a comprehensive audit trail of all operations performed in your Metronome
        account, whether initiated through the API, web interface, or automated
        processes. This endpoint provides detailed logs of who did what and when,
        enabling compliance reporting, security monitoring, and operational
        troubleshooting across all interaction channels.

        ### Use this endpoint to:

        - Monitor all account activity for security and compliance purposes
        - Track configuration changes regardless of source (API, UI, or system)
        - Investigate issues by reviewing historical operations

        ### Key response fields:

        An array of AuditLog objects containing:

        - id: Unique identifier for the audit log entry
        - timestamp: When the action occurred (RFC 3339 format)
        - actor: Information about who performed the action
        - request: Details including request ID, IP address, and user agent
        - `resource_type`: The type of resource affected (e.g., customer, contract,
          invoice)
        - `resource_id`: The specific resource identifier
        - `action`: The operation performed
        - `next_page`: Cursor for continuous log retrieval

        ### Usage guidelines:

        - Continuous retrieval: The next_page token enables uninterrupted log
          streaming—save it between requests to ensure no logs are missed
        - Empty responses: An empty data array means no new logs yet; continue polling
          with the same next_page token
        - Date filtering:
          - `starting_on`: Earliest logs to return (inclusive)
          - `ending_before`: Latest logs to return (exclusive)
          - Cannot be used with `next_page`
        - Resource filtering: Must specify both `resource_type` and `resource_id`
          together
        - Sort order: Default is `date_asc`; use `date_desc` for newest first

        Args:
          ending_before: RFC 3339 timestamp (exclusive). Cannot be used with 'next_page'.

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          resource_id: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_id, you must also specify resource_type.

          resource_type: Optional parameter that can be used to filter which audit logs are returned. If
              you specify resource_type, you must also specify resource_id.

          sort: Sort order by timestamp, e.g. date_asc or date_desc. Defaults to date_asc.

          starting_on: RFC 3339 timestamp of the earliest audit log to return. Cannot be used with
              'next_page'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/auditLogs",
            page=AsyncCursorPage[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "next_page": next_page,
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "sort": sort,
                        "starting_on": starting_on,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
            ),
            model=AuditLogListResponse,
        )


class AuditLogsResourceWithRawResponse:
    def __init__(self, audit_logs: AuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = to_raw_response_wrapper(
            audit_logs.list,
        )


class AsyncAuditLogsResourceWithRawResponse:
    def __init__(self, audit_logs: AsyncAuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_raw_response_wrapper(
            audit_logs.list,
        )


class AuditLogsResourceWithStreamingResponse:
    def __init__(self, audit_logs: AuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = to_streamed_response_wrapper(
            audit_logs.list,
        )


class AsyncAuditLogsResourceWithStreamingResponse:
    def __init__(self, audit_logs: AsyncAuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_streamed_response_wrapper(
            audit_logs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .tier import Tier
from ..._models import BaseModel
from .override_tier import OverrideTier
from .overwrite_rate import OverwriteRate
from .credit_type_data import CreditTypeData

__all__ = ["Override", "OverrideSpecifier", "Product"]


class OverrideSpecifier(BaseModel):
    billing_frequency: Optional[Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]] = None

    commit_ids: Optional[List[str]] = None

    presentation_group_values: Optional[Dict[str, Optional[str]]] = None

    pricing_group_values: Optional[Dict[str, str]] = None

    product_id: Optional[str] = None

    product_tags: Optional[List[str]] = None

    recurring_commit_ids: Optional[List[str]] = None

    recurring_credit_ids: Optional[List[str]] = None


class Product(BaseModel):
    id: str

    name: str


class Override(BaseModel):
    id: str

    starting_at: datetime

    applicable_product_tags: Optional[List[str]] = None

    credit_type: Optional[CreditTypeData] = None

    ending_before: Optional[datetime] = None

    entitled: Optional[bool] = None

    is_commit_specific: Optional[bool] = None

    is_prorated: Optional[bool] = None
    """Default proration configuration.

    Only valid for SUBSCRIPTION rate_type. Must be set to true.
    """

    multiplier: Optional[float] = None

    override_specifiers: Optional[List[OverrideSpecifier]] = None

    override_tiers: Optional[List[OverrideTier]] = None

    overwrite_rate: Optional[OverwriteRate] = None

    price: Optional[float] = None
    """Default price.

    For FLAT rate_type, this must be >=0. For PERCENTAGE rate_type, this is a
    decimal fraction, e.g. use 0.1 for 10%; this must be >=0 and <=1.
    """

    priority: Optional[float] = None

    product: Optional[Product] = None

    quantity: Optional[float] = None
    """Default quantity. For SUBSCRIPTION rate_type, this must be >=0."""

    rate_type: Optional[Literal["FLAT", "PERCENTAGE", "SUBSCRIPTION", "TIERED", "CUSTOM"]] = None

    target: Optional[Literal["COMMIT_RATE", "LIST_RATE"]] = None

    tiers: Optional[List[Tier]] = None
    """Only set for TIERED rate_type."""

    type: Optional[Literal["OVERWRITE", "MULTIPLIER", "TIERED"]] = None

    value: Optional[Dict[str, object]] = None
    """Only set for CUSTOM rate_type.

    This field is interpreted by custom rate processors.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel
from ....shared.id import ID

__all__ = ["RateAddManyResponse"]


class RateAddManyResponse(BaseModel):
    data: ID
    """The ID of the rate card to which the rates were added."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .id import ID as ID
from .rate import Rate as Rate
from .tier import Tier as Tier
from .commit import Commit as Commit
from .credit import Credit as Credit
from .contract import Contract as Contract
from .discount import Discount as Discount
from .override import Override as Override
from .commit_rate import CommitRate as CommitRate
from .contract_v2 import ContractV2 as ContractV2
from .pro_service import ProService as ProService
from .subscription import Subscription as Subscription
from .override_tier import OverrideTier as OverrideTier
from .overwrite_rate import OverwriteRate as OverwriteRate
from .property_filter import PropertyFilter as PropertyFilter
from .commit_specifier import CommitSpecifier as CommitSpecifier
from .credit_type_data import CreditTypeData as CreditTypeData
from .scheduled_charge import ScheduledCharge as ScheduledCharge
from .base_usage_filter import BaseUsageFilter as BaseUsageFilter
from .event_type_filter import EventTypeFilter as EventTypeFilter
from .schedule_duration import ScheduleDuration as ScheduleDuration
from .payment_gate_config import PaymentGateConfig as PaymentGateConfig
from .base_threshold_commit import BaseThresholdCommit as BaseThresholdCommit
from .commit_specifier_input import CommitSpecifierInput as CommitSpecifierInput
from .payment_gate_config_v2 import PaymentGateConfigV2 as PaymentGateConfigV2
from .schedule_point_in_time import SchedulePointInTime as SchedulePointInTime
from .hierarchy_configuration import HierarchyConfiguration as HierarchyConfiguration
from .contract_without_amendments import ContractWithoutAmendments as ContractWithoutAmendments
from .update_base_threshold_commit import UpdateBaseThresholdCommit as UpdateBaseThresholdCommit
from .spend_threshold_configuration import SpendThresholdConfiguration as SpendThresholdConfiguration
from .commit_hierarchy_configuration import CommitHierarchyConfiguration as CommitHierarchyConfiguration
from .spend_threshold_configuration_v2 import SpendThresholdConfigurationV2 as SpendThresholdConfigurationV2
from .recurring_commit_subscription_config import RecurringCommitSubscriptionConfig as RecurringCommitSubscriptionConfig
from .prepaid_balance_threshold_configuration import (
    PrepaidBalanceThresholdConfiguration as PrepaidBalanceThresholdConfiguration,
)
from .prepaid_balance_threshold_configuration_v2 import (
    PrepaidBalanceThresholdConfigurationV2 as PrepaidBalanceThresholdConfigurationV2,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1 import SettingUpsertAvalaraCredentialsResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSettings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_upsert_avalara_credentials(self, client: Metronome) -> None:
        setting = client.v1.settings.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
        )
        assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

    @parametrize
    def test_method_upsert_avalara_credentials_with_all_params(self, client: Metronome) -> None:
        setting = client.v1.settings.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
            commit_transactions=True,
        )
        assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

    @parametrize
    def test_raw_response_upsert_avalara_credentials(self, client: Metronome) -> None:
        response = client.v1.settings.with_raw_response.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        setting = response.parse()
        assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

    @parametrize
    def test_streaming_response_upsert_avalara_credentials(self, client: Metronome) -> None:
        with client.v1.settings.with_streaming_response.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            setting = response.parse()
            assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSettings:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_upsert_avalara_credentials(self, async_client: AsyncMetronome) -> None:
        setting = await async_client.v1.settings.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
        )
        assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

    @parametrize
    async def test_method_upsert_avalara_credentials_with_all_params(self, async_client: AsyncMetronome) -> None:
        setting = await async_client.v1.settings.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
            commit_transactions=True,
        )
        assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

    @parametrize
    async def test_raw_response_upsert_avalara_credentials(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.settings.with_raw_response.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        setting = await response.parse()
        assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

    @parametrize
    async def test_streaming_response_upsert_avalara_credentials(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.settings.with_streaming_response.upsert_avalara_credentials(
            avalara_environment="PRODUCTION",
            avalara_password="my_password_123",
            avalara_username="test@metronome.com",
            delivery_method_ids=["9a906ebb-fbc7-42e8-8e29-53bfd2db3aca"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            setting = await response.parse()
            assert_matches_type(SettingUpsertAvalaraCredentialsResponse, setting, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import (
    ID as ID,
    Rate as Rate,
    Tier as Tier,
    Commit as Commit,
    Credit as Credit,
    Contract as Contract,
    Discount as Discount,
    Override as Override,
    CommitRate as CommitRate,
    ContractV2 as ContractV2,
    ProService as ProService,
    OverrideTier as OverrideTier,
    Subscription as Subscription,
    OverwriteRate as OverwriteRate,
    CreditTypeData as CreditTypeData,
    PropertyFilter as PropertyFilter,
    BaseUsageFilter as BaseUsageFilter,
    CommitSpecifier as CommitSpecifier,
    EventTypeFilter as EventTypeFilter,
    ScheduledCharge as ScheduledCharge,
    ScheduleDuration as ScheduleDuration,
    PaymentGateConfig as PaymentGateConfig,
    BaseThresholdCommit as BaseThresholdCommit,
    PaymentGateConfigV2 as PaymentGateConfigV2,
    SchedulePointInTime as SchedulePointInTime,
    CommitSpecifierInput as CommitSpecifierInput,
    HierarchyConfiguration as HierarchyConfiguration,
    ContractWithoutAmendments as ContractWithoutAmendments,
    UpdateBaseThresholdCommit as UpdateBaseThresholdCommit,
    SpendThresholdConfiguration as SpendThresholdConfiguration,
    CommitHierarchyConfiguration as CommitHierarchyConfiguration,
    SpendThresholdConfigurationV2 as SpendThresholdConfigurationV2,
    RecurringCommitSubscriptionConfig as RecurringCommitSubscriptionConfig,
    PrepaidBalanceThresholdConfiguration as PrepaidBalanceThresholdConfiguration,
    PrepaidBalanceThresholdConfigurationV2 as PrepaidBalanceThresholdConfigurationV2,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1 import AlertCreateResponse, AlertArchiveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAlerts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Metronome) -> None:
        alert = client.v1.alerts.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
        )
        assert_matches_type(AlertCreateResponse, alert, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Metronome) -> None:
        alert = client.v1.alerts.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
            billable_metric_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            credit_grant_type_filters=["enterprise"],
            credit_type_id="2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            custom_field_filters=[
                {
                    "entity": "Contract",
                    "key": "key",
                    "value": "value",
                }
            ],
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            evaluate_on_create=True,
            group_values=[
                {
                    "key": "key",
                    "value": "value",
                }
            ],
            invoice_types_filter=["SCHEDULED or USAGE"],
            plan_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            seat_filter={
                "seat_group_key": "seat_group_key",
                "seat_group_value": "seat_group_value",
            },
            uniqueness_key="x",
        )
        assert_matches_type(AlertCreateResponse, alert, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Metronome) -> None:
        response = client.v1.alerts.with_raw_response.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(AlertCreateResponse, alert, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Metronome) -> None:
        with client.v1.alerts.with_streaming_response.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(AlertCreateResponse, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_archive(self, client: Metronome) -> None:
        alert = client.v1.alerts.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )
        assert_matches_type(AlertArchiveResponse, alert, path=["response"])

    @parametrize
    def test_method_archive_with_all_params(self, client: Metronome) -> None:
        alert = client.v1.alerts.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
            release_uniqueness_key=True,
        )
        assert_matches_type(AlertArchiveResponse, alert, path=["response"])

    @parametrize
    def test_raw_response_archive(self, client: Metronome) -> None:
        response = client.v1.alerts.with_raw_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(AlertArchiveResponse, alert, path=["response"])

    @parametrize
    def test_streaming_response_archive(self, client: Metronome) -> None:
        with client.v1.alerts.with_streaming_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(AlertArchiveResponse, alert, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAlerts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncMetronome) -> None:
        alert = await async_client.v1.alerts.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
        )
        assert_matches_type(AlertCreateResponse, alert, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMetronome) -> None:
        alert = await async_client.v1.alerts.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
            billable_metric_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            credit_grant_type_filters=["enterprise"],
            credit_type_id="2714e483-4ff1-48e4-9e25-ac732e8f24f2",
            custom_field_filters=[
                {
                    "entity": "Contract",
                    "key": "key",
                    "value": "value",
                }
            ],
            customer_id="4db51251-61de-4bfe-b9ce-495e244f3491",
            evaluate_on_create=True,
            group_values=[
                {
                    "key": "key",
                    "value": "value",
                }
            ],
            invoice_types_filter=["SCHEDULED or USAGE"],
            plan_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            seat_filter={
                "seat_group_key": "seat_group_key",
                "seat_group_value": "seat_group_value",
            },
            uniqueness_key="x",
        )
        assert_matches_type(AlertCreateResponse, alert, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.alerts.with_raw_response.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = await response.parse()
        assert_matches_type(AlertCreateResponse, alert, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.alerts.with_streaming_response.create(
            alert_type="spend_threshold_reached",
            name="$100 spend threshold reached",
            threshold=10000,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(AlertCreateResponse, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_archive(self, async_client: AsyncMetronome) -> None:
        alert = await async_client.v1.alerts.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )
        assert_matches_type(AlertArchiveResponse, alert, path=["response"])

    @parametrize
    async def test_method_archive_with_all_params(self, async_client: AsyncMetronome) -> None:
        alert = await async_client.v1.alerts.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
            release_uniqueness_key=True,
        )
        assert_matches_type(AlertArchiveResponse, alert, path=["response"])

    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.alerts.with_raw_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = await response.parse()
        assert_matches_type(AlertArchiveResponse, alert, path=["response"])

    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.alerts.with_streaming_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(AlertArchiveResponse, alert, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .payment_gate_config import PaymentGateConfig
from .base_threshold_commit import BaseThresholdCommit

__all__ = ["SpendThresholdConfiguration"]


class SpendThresholdConfiguration(TypedDict, total=False):
    commit: Required[BaseThresholdCommit]

    is_enabled: Required[bool]
    """
    When set to false, the contract will not be evaluated against the
    threshold_amount. Toggling to true will result an immediate evaluation,
    regardless of prior state.
    """

    payment_gate_config: Required[PaymentGateConfig]

    threshold_amount: Required[float]
    """Specify the threshold amount for the contract.

    Each time the contract's usage hits this amount, a threshold charge will be
    initiated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from .credit_type_data import CreditTypeData

__all__ = ["ScheduleDuration", "ScheduleItem"]


class ScheduleItem(BaseModel):
    id: str

    amount: float

    ending_before: datetime

    starting_at: datetime


class ScheduleDuration(BaseModel):
    schedule_items: List[ScheduleItem]

    credit_type: Optional[CreditTypeData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PricingUnitListResponse"]


class PricingUnitListResponse(BaseModel):
    id: Optional[str] = None

    is_currency: Optional[bool] = None

    name: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContractListParams"]


class ContractListParams(TypedDict, total=False):
    customer_id: Required[str]

    covering_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Optional RFC 3339 timestamp.

    If provided, the response will include only contracts effective on the provided
    date. This cannot be provided if the starting_at filter is provided.
    """

    include_archived: bool
    """Include archived contracts in the response"""

    include_balance: bool
    """Include the balance of credits and commits in the response.

    Setting this flag may cause the query to be slower.
    """

    include_ledgers: bool
    """Include commit ledgers in the response.

    Setting this flag may cause the query to be slower.
    """

    starting_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Optional RFC 3339 timestamp.

    If provided, the response will include only contracts where effective_at is on
    or after the provided date. This cannot be provided if the covering_date filter
    is provided.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["RateCardRetrieveRateScheduleParams", "Selector"]


class RateCardRetrieveRateScheduleParams(TypedDict, total=False):
    rate_card_id: Required[str]
    """ID of the rate card to get the schedule for"""

    starting_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """inclusive starting point for the rates schedule"""

    limit: int
    """Max number of results that should be returned"""

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    ending_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """optional exclusive end date for the rates schedule.

    When not specified rates will show all future schedule segments.
    """

    selectors: Iterable[Selector]
    """
    List of rate selectors, rates matching ANY of the selector will be included in
    the response Passing no selectors will result in all rates being returned.
    """


class Selector(TypedDict, total=False):
    billing_frequency: Literal["MONTHLY", "QUARTERLY", "ANNUAL", "WEEKLY"]
    """
    Subscription rates matching the billing frequency will be included in the
    response.
    """

    partial_pricing_group_values: Dict[str, str]
    """
    List of pricing group key value pairs, rates containing the matching key / value
    pairs will be included in the response.
    """

    pricing_group_values: Dict[str, str]
    """
    List of pricing group key value pairs, rates matching all of the key / value
    pairs will be included in the response.
    """

    product_id: str
    """Rates matching the product id will be included in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = [
    "SyncCursorPage",
    "AsyncCursorPage",
    "SyncBodyCursorPage",
    "AsyncBodyCursorPage",
    "SyncCursorPageWithoutLimit",
    "AsyncCursorPageWithoutLimit",
]

_T = TypeVar("_T")


class SyncCursorPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    next_page: Optional[str] = None
    """Cursor to fetch the next page"""
    data: List[_T]
    """Items of the page"""

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def has_next_page(self) -> bool:
        return self.next_page_info() is not None

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page = self.next_page
        if not next_page:
            return None

        return PageInfo(params={"next_page": next_page})


class AsyncCursorPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    next_page: Optional[str] = None
    """Cursor to fetch the next page"""
    data: List[_T]
    """Items of the page"""

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def has_next_page(self) -> bool:
        return self.next_page_info() is not None

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page = self.next_page
        if not next_page:
            return None

        return PageInfo(params={"next_page": next_page})


class SyncBodyCursorPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    next_page: Optional[str] = None
    """Cursor to fetch the next page"""
    data: List[_T]
    """Items of the page"""

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def has_next_page(self) -> bool:
        return self.next_page_info() is not None

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page = self.next_page
        if not next_page:
            return None

        return PageInfo(json={"next_page": next_page})


class AsyncBodyCursorPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    next_page: Optional[str] = None
    """Cursor to fetch the next page"""
    data: List[_T]
    """Items of the page"""

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def has_next_page(self) -> bool:
        return self.next_page_info() is not None

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page = self.next_page
        if not next_page:
            return None

        return PageInfo(json={"next_page": next_page})


class SyncCursorPageWithoutLimit(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    next_page: Optional[str] = None
    """Cursor to fetch the next page"""
    data: List[_T]
    """Items of the page"""

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def has_next_page(self) -> bool:
        return self.next_page_info() is not None

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page = self.next_page
        if not next_page:
            return None

        return PageInfo(params={"next_page": next_page})


class AsyncCursorPageWithoutLimit(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    next_page: Optional[str] = None
    """Cursor to fetch the next page"""
    data: List[_T]
    """Items of the page"""

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def has_next_page(self) -> bool:
        return self.next_page_info() is not None

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_page = self.next_page
        if not next_page:
            return None

        return PageInfo(params={"next_page": next_page})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .tier import Tier

__all__ = ["CommitRate"]


class CommitRate(TypedDict, total=False):
    """A distinct rate on the rate card.

    You can choose to use this rate rather than list rate when consuming a credit or commit.
    """

    rate_type: Required[Literal["FLAT", "PERCENTAGE", "SUBSCRIPTION", "TIERED", "CUSTOM"]]

    price: float
    """Commit rate price. For FLAT rate_type, this must be >=0."""

    tiers: Iterable[Tier]
    """Only set for TIERED rate_type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1 import (
    BillableMetricListResponse,
    BillableMetricCreateResponse,
    BillableMetricArchiveResponse,
    BillableMetricRetrieveResponse,
)
from metronome.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBillableMetrics:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Metronome) -> None:
        billable_metric = client.v1.billable_metrics.create(
            name="CPU Hours",
        )
        assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Metronome) -> None:
        billable_metric = client.v1.billable_metrics.create(
            name="CPU Hours",
            aggregation_key="cpu_hours",
            aggregation_type="SUM",
            custom_fields={"foo": "string"},
            event_type_filter={
                "in_values": ["cpu_usage"],
                "not_in_values": ["string"],
            },
            group_keys=[["region"], ["machine_type"]],
            property_filters=[
                {
                    "name": "cpu_hours",
                    "exists": True,
                    "in_values": ["string"],
                    "not_in_values": ["string"],
                },
                {
                    "name": "region",
                    "exists": True,
                    "in_values": ["EU", "NA"],
                    "not_in_values": ["string"],
                },
                {
                    "name": "machine_type",
                    "exists": True,
                    "in_values": ["slow", "fast"],
                    "not_in_values": ["string"],
                },
            ],
            sql="sql",
        )
        assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Metronome) -> None:
        response = client.v1.billable_metrics.with_raw_response.create(
            name="CPU Hours",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = response.parse()
        assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Metronome) -> None:
        with client.v1.billable_metrics.with_streaming_response.create(
            name="CPU Hours",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = response.parse()
            assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Metronome) -> None:
        billable_metric = client.v1.billable_metrics.retrieve(
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(BillableMetricRetrieveResponse, billable_metric, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Metronome) -> None:
        response = client.v1.billable_metrics.with_raw_response.retrieve(
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = response.parse()
        assert_matches_type(BillableMetricRetrieveResponse, billable_metric, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Metronome) -> None:
        with client.v1.billable_metrics.with_streaming_response.retrieve(
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = response.parse()
            assert_matches_type(BillableMetricRetrieveResponse, billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `billable_metric_id` but received ''"):
            client.v1.billable_metrics.with_raw_response.retrieve(
                billable_metric_id="",
            )

    @parametrize
    def test_method_list(self, client: Metronome) -> None:
        billable_metric = client.v1.billable_metrics.list()
        assert_matches_type(SyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Metronome) -> None:
        billable_metric = client.v1.billable_metrics.list(
            include_archived=True,
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(SyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Metronome) -> None:
        response = client.v1.billable_metrics.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = response.parse()
        assert_matches_type(SyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Metronome) -> None:
        with client.v1.billable_metrics.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = response.parse()
            assert_matches_type(SyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_archive(self, client: Metronome) -> None:
        billable_metric = client.v1.billable_metrics.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )
        assert_matches_type(BillableMetricArchiveResponse, billable_metric, path=["response"])

    @parametrize
    def test_raw_response_archive(self, client: Metronome) -> None:
        response = client.v1.billable_metrics.with_raw_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = response.parse()
        assert_matches_type(BillableMetricArchiveResponse, billable_metric, path=["response"])

    @parametrize
    def test_streaming_response_archive(self, client: Metronome) -> None:
        with client.v1.billable_metrics.with_streaming_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = response.parse()
            assert_matches_type(BillableMetricArchiveResponse, billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBillableMetrics:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncMetronome) -> None:
        billable_metric = await async_client.v1.billable_metrics.create(
            name="CPU Hours",
        )
        assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMetronome) -> None:
        billable_metric = await async_client.v1.billable_metrics.create(
            name="CPU Hours",
            aggregation_key="cpu_hours",
            aggregation_type="SUM",
            custom_fields={"foo": "string"},
            event_type_filter={
                "in_values": ["cpu_usage"],
                "not_in_values": ["string"],
            },
            group_keys=[["region"], ["machine_type"]],
            property_filters=[
                {
                    "name": "cpu_hours",
                    "exists": True,
                    "in_values": ["string"],
                    "not_in_values": ["string"],
                },
                {
                    "name": "region",
                    "exists": True,
                    "in_values": ["EU", "NA"],
                    "not_in_values": ["string"],
                },
                {
                    "name": "machine_type",
                    "exists": True,
                    "in_values": ["slow", "fast"],
                    "not_in_values": ["string"],
                },
            ],
            sql="sql",
        )
        assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.billable_metrics.with_raw_response.create(
            name="CPU Hours",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = await response.parse()
        assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.billable_metrics.with_streaming_response.create(
            name="CPU Hours",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = await response.parse()
            assert_matches_type(BillableMetricCreateResponse, billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMetronome) -> None:
        billable_metric = await async_client.v1.billable_metrics.retrieve(
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )
        assert_matches_type(BillableMetricRetrieveResponse, billable_metric, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.billable_metrics.with_raw_response.retrieve(
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = await response.parse()
        assert_matches_type(BillableMetricRetrieveResponse, billable_metric, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.billable_metrics.with_streaming_response.retrieve(
            billable_metric_id="13117714-3f05-48e5-a6e9-a66093f13b4d",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = await response.parse()
            assert_matches_type(BillableMetricRetrieveResponse, billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `billable_metric_id` but received ''"):
            await async_client.v1.billable_metrics.with_raw_response.retrieve(
                billable_metric_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncMetronome) -> None:
        billable_metric = await async_client.v1.billable_metrics.list()
        assert_matches_type(AsyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMetronome) -> None:
        billable_metric = await async_client.v1.billable_metrics.list(
            include_archived=True,
            limit=1,
            next_page="next_page",
        )
        assert_matches_type(AsyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.billable_metrics.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = await response.parse()
        assert_matches_type(AsyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.billable_metrics.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = await response.parse()
            assert_matches_type(AsyncCursorPage[BillableMetricListResponse], billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_archive(self, async_client: AsyncMetronome) -> None:
        billable_metric = await async_client.v1.billable_metrics.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )
        assert_matches_type(BillableMetricArchiveResponse, billable_metric, path=["response"])

    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.billable_metrics.with_raw_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billable_metric = await response.parse()
        assert_matches_type(BillableMetricArchiveResponse, billable_metric, path=["response"])

    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.billable_metrics.with_streaming_response.archive(
            id="8deed800-1b7a-495d-a207-6c52bac54dc9",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billable_metric = await response.parse()
            assert_matches_type(BillableMetricArchiveResponse, billable_metric, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["BillingProviderListParams"]


class BillingProviderListParams(TypedDict, total=False):
    next_page: Optional[str]
    """The cursor to the next page of results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ..._models import BaseModel

__all__ = ["Customer"]


class Customer(BaseModel):
    id: str
    """the Metronome ID of the customer"""

    external_id: str
    """
    (deprecated, use ingest_aliases instead) the first ID (Metronome or ingest
    alias) that can be used in usage events
    """

    ingest_aliases: List[str]
    """
    aliases for this customer that can be used instead of the Metronome customer ID
    in usage events
    """

    name: str

    custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel
from ...shared.rate import Rate
from ...shared.credit_type_data import CreditTypeData

__all__ = [
    "Invoice",
    "LineItem",
    "LineItemAppliedCommitOrCredit",
    "LineItemOrigin",
    "LineItemPostpaidCommit",
    "LineItemSubLineItem",
    "LineItemSubLineItemTierPeriod",
    "LineItemSubLineItemTier",
    "LineItemTier",
    "ConstituentInvoice",
    "CorrectionRecord",
    "CorrectionRecordCorrectedExternalInvoice",
    "CorrectionRecordCorrectedExternalInvoiceTax",
    "ExternalInvoice",
    "ExternalInvoiceTax",
    "InvoiceAdjustment",
    "Payer",
    "ResellerRoyalty",
    "ResellerRoyaltyAwsOptions",
    "ResellerRoyaltyGcpOptions",
]


class LineItemAppliedCommitOrCredit(BaseModel):
    """Details about the credit or commit that was applied to this line item.

    Only present on line items with product of `USAGE`, `SUBSCRIPTION` or `COMPOSITE` types.
    """

    id: str

    type: Literal["PREPAID", "POSTPAID", "CREDIT"]


class LineItemOrigin(BaseModel):
    """
    Account hierarchy M3 - Present on line items from invoices with type USAGE_CONSOLIDATED. Indicates the original customer, contract, invoice and line item from which this line item was copied.
    """

    contract_id: str

    customer_id: str

    invoice_id: str

    line_item_id: str


class LineItemPostpaidCommit(BaseModel):
    """Only present for line items paying for a postpaid commit true-up."""

    id: str


class LineItemSubLineItemTierPeriod(BaseModel):
    """when the current tier started and ends (for tiered charges only)"""

    starting_at: datetime

    ending_before: Optional[datetime] = None


class LineItemSubLineItemTier(BaseModel):
    price: float

    quantity: float

    starting_at: float
    """at what metric amount this tier begins"""

    subtotal: float


class LineItemSubLineItem(BaseModel):
    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    name: str

    quantity: float

    subtotal: float

    charge_id: Optional[str] = None

    credit_grant_id: Optional[str] = None

    end_date: Optional[datetime] = None
    """The end date for the charge (for seats charges only)."""

    price: Optional[float] = None
    """
    the unit price for this charge, present only if the charge is not tiered and the
    quantity is nonzero
    """

    start_date: Optional[datetime] = None
    """The start date for the charge (for seats charges only)."""

    tier_period: Optional[LineItemSubLineItemTierPeriod] = None
    """when the current tier started and ends (for tiered charges only)"""

    tiers: Optional[List[LineItemSubLineItemTier]] = None


class LineItemTier(BaseModel):
    """Populated if the line item has a tiered price."""

    level: float

    starting_at: str

    size: Optional[str] = None


class LineItem(BaseModel):
    credit_type: CreditTypeData

    name: str

    total: float

    type: str
    """The type of line item.

    - `scheduled`: Line item is associated with a scheduled charge. View the
      scheduled_charge_id on the line item.
    - `commit_purchase`: Line item is associated with a payment for a prepaid
      commit. View the commit_id on the line item.
    - `usage`: Line item is associated with a usage product or composite product.
      View the product_id on the line item to determine which product.
    - `subscription`: Line item is associated with a subscription. e.g. monthly
      recurring payment for an in-advance subscription.
    - `applied_commit_or_credit`: On metronome invoices, applied commits and credits
      are associated with their own line items. These line items have negative
      totals. Use the applied_commit_or_credit object on the line item to understand
      the id of the applied commit or credit, and its type. Note that the
      application of a postpaid commit is associated with a line item, but the total
      on the line item is not included in the invoice's total as postpaid commits
      are paid in-arrears.
    - `cpu_conversion`: Line item converting between a custom pricing unit and fiat
      currency, using the conversion rate set on the rate card. This line item will
      appear when there are products priced in custom pricing units, and there is
      insufficient prepaid commit/credit in that custom pricing unit to fully cover
      the spend. Then, the outstanding spend in custom pricing units will be
      converted to fiat currency using a cpu_conversion line item.
    """

    applied_commit_or_credit: Optional[LineItemAppliedCommitOrCredit] = None
    """Details about the credit or commit that was applied to this line item.

    Only present on line items with product of `USAGE`, `SUBSCRIPTION` or
    `COMPOSITE` types.
    """

    commit_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    commit_id: Optional[str] = None
    """
    For line items with product of `USAGE`, `SUBSCRIPTION`, or `COMPOSITE` types,
    the ID of the credit or commit that was applied to this line item. For line
    items with product type of `FIXED`, the ID of the prepaid or postpaid commit
    that is being paid for.
    """

    commit_netsuite_item_id: Optional[str] = None

    commit_netsuite_sales_order_id: Optional[str] = None

    commit_segment_id: Optional[str] = None

    commit_type: Optional[str] = None
    """
    `PrepaidCommit` (for commit types `PREPAID` and `CREDIT`) or `PostpaidCommit`
    (for commit type `POSTPAID`).
    """

    custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    discount_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    discount_id: Optional[str] = None
    """ID of the discount applied to this line item."""

    ending_before: Optional[datetime] = None
    """The line item's end date (exclusive)."""

    group_key: Optional[str] = None

    group_value: Optional[str] = None

    is_prorated: Optional[bool] = None
    """Indicates whether the line item is prorated for `SUBSCRIPTION` type product."""

    list_price: Optional[Rate] = None
    """
    Only present for contract invoices and when the `include_list_prices` query
    parameter is set to true. This will include the list rate for the charge if
    applicable. Only present for usage and subscription line items.
    """

    metadata: Optional[str] = None

    netsuite_invoice_billing_end: Optional[datetime] = None
    """The end date for the billing period on the invoice."""

    netsuite_invoice_billing_start: Optional[datetime] = None
    """The start date for the billing period on the invoice."""

    netsuite_item_id: Optional[str] = None

    origin: Optional[LineItemOrigin] = None
    """
    Account hierarchy M3 - Present on line items from invoices with type
    USAGE_CONSOLIDATED. Indicates the original customer, contract, invoice and line
    item from which this line item was copied.
    """

    postpaid_commit: Optional[LineItemPostpaidCommit] = None
    """Only present for line items paying for a postpaid commit true-up."""

    presentation_group_values: Optional[Dict[str, Optional[str]]] = None
    """
    Includes the presentation group values associated with this line item if
    presentation group keys are used.
    """

    pricing_group_values: Optional[Dict[str, str]] = None
    """
    Includes the pricing group values associated with this line item if dimensional
    pricing is used.
    """

    product_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    product_id: Optional[str] = None
    """ID of the product associated with the line item."""

    product_tags: Optional[List[str]] = None
    """The current product tags associated with the line item's `product_id`."""

    product_type: Optional[str] = None
    """The type of the line item's product.

    Possible values are `FixedProductListItem` (for `FIXED` type products),
    `UsageProductListItem` (for `USAGE` type products),
    `SubscriptionProductListItem` (for `SUBSCRIPTION` type products) or
    `CompositeProductListItem` (for `COMPOSITE` type products). For scheduled
    charges, commit and credit payments, the value is `FixedProductListItem`.
    """

    professional_service_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    professional_service_id: Optional[str] = None

    quantity: Optional[float] = None
    """The quantity associated with the line item."""

    reseller_type: Optional[Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]] = None

    scheduled_charge_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    scheduled_charge_id: Optional[str] = None
    """ID of scheduled charge."""

    starting_at: Optional[datetime] = None
    """The line item's start date (inclusive)."""

    sub_line_items: Optional[List[LineItemSubLineItem]] = None

    subscription_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    tier: Optional[LineItemTier] = None
    """Populated if the line item has a tiered price."""

    unit_price: Optional[float] = None
    """The unit price associated with the line item."""


class ConstituentInvoice(BaseModel):
    contract_id: str

    customer_id: str

    invoice_id: str


class CorrectionRecordCorrectedExternalInvoiceTax(BaseModel):
    """Tax details for the invoice, if available from the billing provider."""

    total_tax_amount: Optional[float] = None
    """The total tax amount applied to the invoice."""

    total_taxable_amount: Optional[float] = None
    """The total taxable amount of the invoice."""

    transaction_id: Optional[str] = None
    """The transaction ID associated with the tax calculation."""


class CorrectionRecordCorrectedExternalInvoice(BaseModel):
    billing_provider_type: Literal[
        "aws_marketplace",
        "stripe",
        "netsuite",
        "custom",
        "azure_marketplace",
        "quickbooks_online",
        "workday",
        "gcp_marketplace",
        "metronome",
    ]

    billing_provider_error: Optional[str] = None
    """Error message from the billing provider, if available."""

    external_payment_id: Optional[str] = None
    """The ID of the payment in the external system, if available."""

    external_status: Optional[
        Literal[
            "DRAFT",
            "FINALIZED",
            "PAID",
            "PARTIALLY_PAID",
            "UNCOLLECTIBLE",
            "VOID",
            "DELETED",
            "PAYMENT_FAILED",
            "INVALID_REQUEST_ERROR",
            "SKIPPED",
            "SENT",
            "QUEUED",
        ]
    ] = None

    invoice_id: Optional[str] = None

    invoiced_sub_total: Optional[float] = None
    """The subtotal amount invoiced, if available from the billing provider."""

    invoiced_total: Optional[float] = None
    """The total amount invoiced, if available from the billing provider."""

    issued_at_timestamp: Optional[datetime] = None

    pdf_url: Optional[str] = None
    """A URL to the PDF of the invoice, if available from the billing provider."""

    tax: Optional[CorrectionRecordCorrectedExternalInvoiceTax] = None
    """Tax details for the invoice, if available from the billing provider."""


class CorrectionRecord(BaseModel):
    corrected_invoice_id: str

    memo: str

    reason: str

    corrected_external_invoice: Optional[CorrectionRecordCorrectedExternalInvoice] = None


class ExternalInvoiceTax(BaseModel):
    """Tax details for the invoice, if available from the billing provider."""

    total_tax_amount: Optional[float] = None
    """The total tax amount applied to the invoice."""

    total_taxable_amount: Optional[float] = None
    """The total taxable amount of the invoice."""

    transaction_id: Optional[str] = None
    """The transaction ID associated with the tax calculation."""


class ExternalInvoice(BaseModel):
    billing_provider_type: Literal[
        "aws_marketplace",
        "stripe",
        "netsuite",
        "custom",
        "azure_marketplace",
        "quickbooks_online",
        "workday",
        "gcp_marketplace",
        "metronome",
    ]

    billing_provider_error: Optional[str] = None
    """Error message from the billing provider, if available."""

    external_payment_id: Optional[str] = None
    """The ID of the payment in the external system, if available."""

    external_status: Optional[
        Literal[
            "DRAFT",
            "FINALIZED",
            "PAID",
            "PARTIALLY_PAID",
            "UNCOLLECTIBLE",
            "VOID",
            "DELETED",
            "PAYMENT_FAILED",
            "INVALID_REQUEST_ERROR",
            "SKIPPED",
            "SENT",
            "QUEUED",
        ]
    ] = None

    invoice_id: Optional[str] = None

    invoiced_sub_total: Optional[float] = None
    """The subtotal amount invoiced, if available from the billing provider."""

    invoiced_total: Optional[float] = None
    """The total amount invoiced, if available from the billing provider."""

    issued_at_timestamp: Optional[datetime] = None

    pdf_url: Optional[str] = None
    """A URL to the PDF of the invoice, if available from the billing provider."""

    tax: Optional[ExternalInvoiceTax] = None
    """Tax details for the invoice, if available from the billing provider."""


class InvoiceAdjustment(BaseModel):
    credit_type: CreditTypeData

    name: str

    total: float

    credit_grant_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    credit_grant_id: Optional[str] = None


class Payer(BaseModel):
    """Account hierarchy M3 - Required for account hierarchy usage invoices.

    An object containing the contract and customer UUIDs that pay for this invoice.
    """

    contract_id: str

    customer_id: str


class ResellerRoyaltyAwsOptions(BaseModel):
    aws_account_number: Optional[str] = None

    aws_offer_id: Optional[str] = None

    aws_payer_reference_id: Optional[str] = None


class ResellerRoyaltyGcpOptions(BaseModel):
    gcp_account_id: Optional[str] = None

    gcp_offer_id: Optional[str] = None


class ResellerRoyalty(BaseModel):
    """Only present for contract invoices with reseller royalties."""

    fraction: str

    netsuite_reseller_id: str

    reseller_type: Literal["AWS", "AWS_PRO_SERVICE", "GCP", "GCP_PRO_SERVICE"]

    aws_options: Optional[ResellerRoyaltyAwsOptions] = None

    gcp_options: Optional[ResellerRoyaltyGcpOptions] = None


class Invoice(BaseModel):
    id: str

    credit_type: CreditTypeData

    customer_id: str

    line_items: List[LineItem]

    status: str

    total: float

    type: str

    amendment_id: Optional[str] = None

    billable_status: Optional[object] = None
    """This field's availability is dependent on your client's configuration."""

    constituent_invoices: Optional[List[ConstituentInvoice]] = None
    """Account hierarchy M3 - Required on invoices with type USAGE_CONSOLIDATED.

    List of constituent invoices that were consolidated to create this invoice.
    """

    contract_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    contract_id: Optional[str] = None

    correction_record: Optional[CorrectionRecord] = None

    created_at: Optional[datetime] = None
    """When the invoice was created (UTC).

    This field is present for correction invoices only.
    """

    custom_fields: Optional[Dict[str, object]] = None

    customer_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    end_timestamp: Optional[datetime] = None
    """End of the usage period this invoice covers (UTC)"""

    external_invoice: Optional[ExternalInvoice] = None

    invoice_adjustments: Optional[List[InvoiceAdjustment]] = None

    issued_at: Optional[datetime] = None
    """When the invoice was issued (UTC)"""

    net_payment_terms_days: Optional[float] = None

    netsuite_sales_order_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    payer: Optional[Payer] = None
    """Account hierarchy M3 - Required for account hierarchy usage invoices.

    An object containing the contract and customer UUIDs that pay for this invoice.
    """

    plan_custom_fields: Optional[Dict[str, str]] = None
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""

    plan_id: Optional[str] = None

    plan_name: Optional[str] = None

    reseller_royalty: Optional[ResellerRoyalty] = None
    """Only present for contract invoices with reseller royalties."""

    salesforce_opportunity_id: Optional[str] = None
    """This field's availability is dependent on your client's configuration."""

    start_timestamp: Optional[datetime] = None
    """Beginning of the usage period this invoice covers (UTC)"""

    subtotal: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel
from ....shared.id import ID

__all__ = ["ProductOrderUpdateResponse"]


class ProductOrderUpdateResponse(BaseModel):
    data: ID



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["BaseThresholdCommit"]


class BaseThresholdCommit(BaseModel):
    product_id: str
    """
    The commit product that will be used to generate the line item for commit
    payment.
    """

    description: Optional[str] = None

    name: Optional[str] = None
    """Specify the name of the line item for the threshold charge.

    If left blank, it will default to the commit product name.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from metronome import Metronome, AsyncMetronome
from tests.utils import assert_matches_type
from metronome.types.v1.customers import BillingConfigRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBillingConfig:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Metronome) -> None:
        billing_config = client.v1.customers.billing_config.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
        )
        assert billing_config is None

    @parametrize
    def test_method_create_with_all_params(self, client: Metronome) -> None:
        billing_config = client.v1.customers.billing_config.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
            aws_customer_account_id="aws_customer_account_id",
            aws_customer_id="aws_customer_id",
            aws_product_code="aws_product_code",
            aws_region="af-south-1",
            stripe_collection_method="charge_automatically",
        )
        assert billing_config is None

    @parametrize
    def test_raw_response_create(self, client: Metronome) -> None:
        response = client.v1.customers.billing_config.with_raw_response.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_config = response.parse()
        assert billing_config is None

    @parametrize
    def test_streaming_response_create(self, client: Metronome) -> None:
        with client.v1.customers.billing_config.with_streaming_response.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_config = response.parse()
            assert billing_config is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.billing_config.with_raw_response.create(
                customer_id="",
                billing_provider_type="stripe",
                billing_provider_customer_id="cus_AJ6y20bjkOOayM",
            )

    @parametrize
    def test_method_retrieve(self, client: Metronome) -> None:
        billing_config = client.v1.customers.billing_config.retrieve(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )
        assert_matches_type(BillingConfigRetrieveResponse, billing_config, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Metronome) -> None:
        response = client.v1.customers.billing_config.with_raw_response.retrieve(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_config = response.parse()
        assert_matches_type(BillingConfigRetrieveResponse, billing_config, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Metronome) -> None:
        with client.v1.customers.billing_config.with_streaming_response.retrieve(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_config = response.parse()
            assert_matches_type(BillingConfigRetrieveResponse, billing_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.billing_config.with_raw_response.retrieve(
                customer_id="",
                billing_provider_type="stripe",
            )

    @parametrize
    def test_method_delete(self, client: Metronome) -> None:
        billing_config = client.v1.customers.billing_config.delete(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )
        assert billing_config is None

    @parametrize
    def test_raw_response_delete(self, client: Metronome) -> None:
        response = client.v1.customers.billing_config.with_raw_response.delete(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_config = response.parse()
        assert billing_config is None

    @parametrize
    def test_streaming_response_delete(self, client: Metronome) -> None:
        with client.v1.customers.billing_config.with_streaming_response.delete(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_config = response.parse()
            assert billing_config is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Metronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.customers.billing_config.with_raw_response.delete(
                customer_id="",
                billing_provider_type="stripe",
            )


class TestAsyncBillingConfig:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncMetronome) -> None:
        billing_config = await async_client.v1.customers.billing_config.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
        )
        assert billing_config is None

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncMetronome) -> None:
        billing_config = await async_client.v1.customers.billing_config.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
            aws_customer_account_id="aws_customer_account_id",
            aws_customer_id="aws_customer_id",
            aws_product_code="aws_product_code",
            aws_region="af-south-1",
            stripe_collection_method="charge_automatically",
        )
        assert billing_config is None

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.billing_config.with_raw_response.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_config = await response.parse()
        assert billing_config is None

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.billing_config.with_streaming_response.create(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
            billing_provider_customer_id="cus_AJ6y20bjkOOayM",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_config = await response.parse()
            assert billing_config is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.billing_config.with_raw_response.create(
                customer_id="",
                billing_provider_type="stripe",
                billing_provider_customer_id="cus_AJ6y20bjkOOayM",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMetronome) -> None:
        billing_config = await async_client.v1.customers.billing_config.retrieve(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )
        assert_matches_type(BillingConfigRetrieveResponse, billing_config, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.billing_config.with_raw_response.retrieve(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_config = await response.parse()
        assert_matches_type(BillingConfigRetrieveResponse, billing_config, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.billing_config.with_streaming_response.retrieve(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_config = await response.parse()
            assert_matches_type(BillingConfigRetrieveResponse, billing_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.billing_config.with_raw_response.retrieve(
                customer_id="",
                billing_provider_type="stripe",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncMetronome) -> None:
        billing_config = await async_client.v1.customers.billing_config.delete(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )
        assert billing_config is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncMetronome) -> None:
        response = await async_client.v1.customers.billing_config.with_raw_response.delete(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        billing_config = await response.parse()
        assert billing_config is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncMetronome) -> None:
        async with async_client.v1.customers.billing_config.with_streaming_response.delete(
            customer_id="d7abd0cd-4ae9-4db7-8676-e986a4ebd8dc",
            billing_provider_type="stripe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            billing_config = await response.parse()
            assert billing_config is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncMetronome) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.customers.billing_config.with_raw_response.delete(
                customer_id="",
                billing_provider_type="stripe",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .customer import Customer
from ..._models import BaseModel

__all__ = ["CustomerCreateResponse"]


class CustomerCreateResponse(BaseModel):
    data: Customer



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["QuantityRounding"]


class QuantityRounding(BaseModel):
    """Optional.

    Only valid for USAGE products. If provided, the quantity will be rounded using the provided rounding method and decimal places. For example, if the method is "round up" and the decimal places is 0, then the quantity will be rounded up to the nearest integer.
    """

    decimal_places: float

    rounding_method: Literal["ROUND_UP", "ROUND_DOWN", "ROUND_HALF_UP"]
